import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import com.ubuntu.click 0.1

Page {
    id: root
    title: i18n.tr("Updates")

    ClickManager {
        id: clickManager

        Component.onCompleted: clickManager.checkUpdates()
    }

    Button {
        id: installAllButton
        text: i18n.tr("Install %1 Update", "Install %1 Updates", clickManager.model.length).arg(clickManager.model.length)
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            topMargin: units.gu(2)
            leftMargin: units.gu(4)
            rightMargin: units.gu(4)
        }

        color: UbuntuColors.orange
        onClicked: {
            for(var i=0; i < clickManager.model.length; i++){
                var item = clickManager.model[i];
                clickManager.startDownload(item.packageName);
            }
        }
    }

    ListView {
        id: updateList
        anchors {
            left: parent.left
            right: parent.right
            top: installAllButton.bottom
            bottom: parent.bottom
            margins: units.gu(2)
        }
        model: clickManager.model
        clip: true

        delegate: ListItem.Subtitled {
            id: listItem
            icon: Qt.resolvedUrl("../img/toolbarIcon@8.png")
            enabled: !modelData.updateState

            Rectangle {
                id: textArea
                color: "transparent"
                anchors.fill: parent
                anchors.topMargin: units.gu(1)

                Column {
                    id: colRight
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.right: parent.right

                    Button {
                        text: i18n.tr("Update")
                        height: textArea.height / 2

                        onClicked: {
                            modelData.selected = true;
                            clickManager.startDownload(modelData.packageName);
                        }
                    }
                    Label {
                        text: "6.5 MB"
                        height: textArea.height / 2
                        anchors.right: parent.right
                    }
                }

                Column {
                    id: colLeft
                    anchors {
                        left: parent.left
                        bottom: parent.bottom
                        top: parent.top
                        right: colRight.left
                        rightMargin: units.gu(1)
                    }

                    Label {
                        anchors {
                            left: parent.left
                            right: parent.right
                        }
                        text: modelData.title
                        color: modelData.updateState ? "gray" : "white"
                        font.bold: true
                        elide: Text.ElideRight
                    }
                    Label {
                        anchors {
                            left: parent.left
                            right: parent.right
                        }
                        text: modelData.remoteVersion ? i18n.tr("Version: ") + modelData.remoteVersion : ""
                        color: "white"
                        visible: !modelData.updateState
                        elide: Text.ElideRight
                    }
                }
            }

            ProgressBar {
                id: progress
                width: textArea.width - colRight.width - units.gu(2)
                height: parent.height / 3
                anchors.left: textArea.left
                anchors.leftMargin: units.gu(1)
                anchors.bottom: textArea.bottom
                anchors.bottomMargin: units.dp(5)
                opacity: modelData.updateState ? 1 : 0
                value: 0
                minimumValue: 0
                maximumValue: 100

                DownloadTracker {
                    id: tracker
                    service: "com.canonical.applications.Downloader"
                    dbusPath: modelData.updateState ? modelData.dbusPath : ""

                    onProgress: {
                        var percentage = parseInt(received * 100 / total);
                        progress.value = percentage;
                    }

                    onFinished: {
                        clickManager.installClick(path);
                    }
                }

                Behavior on opacity { PropertyAnimation { duration: UbuntuAnimation.SleepyDuration } }
            }
        }
    }
}
