#ifndef CLICKMANAGER_H
#define CLICKMANAGER_H

#include <QQuickItem>
#include <QHash>
#include <QList>
#include <QVariant>
#include <QVariantList>
#include "application.h"

#ifdef TESTS
#include "tests/fakeprocess.h"
#include "tests/fakenetwork.h"
#include "tests/fakedownloader.h"
#include "tests/testclickmanager.h"
#else
#include <QProcess>
#include "network/network.h"
#include "download/downloader.h"
#endif

class ClickManager : public QQuickItem
{
    Q_OBJECT
    Q_DISABLE_COPY(ClickManager)
    Q_PROPERTY(bool updatesAvailable READ updatesAvailable NOTIFY updatesAvailableChanged)
    Q_PROPERTY(QVariantList model READ model NOTIFY modelChanged)

#ifdef TESTS
    friend class TestClickManager;
#endif

signals:
    void updatesAvailableChanged();
    void modelChanged();
    
public:
    ClickManager(QQuickItem *parent = 0);
    ~ClickManager();

    Q_INVOKABLE void checkUpdates();
    Q_INVOKABLE void startDownload(QString packagename);
    Q_INVOKABLE void installClick(QString path);

    bool updatesAvailable() { return this->m_updatesAvailable; }
    QVariantList model() const { return this->m_model; }

private slots:
    void processOutput();
    void processUpdates();
    void downloadUrlObtained(QString packagename, QString url);
    void downloadCreated(QString packagename, QString dbuspath);

private:
    bool m_updatesAvailable;
    QHash<QString, Application*> m_apps;
    QVariantList m_model;
#ifdef TESTS
    FakeNetwork network;
    FakeDownloader downloader;
    FakeProcess m_process;
#else
    QProcess m_process;
    Network network;
    Downloader downloader;
#endif

    void checkForUpdates();
};

QML_DECLARE_TYPE(ClickManager)

#endif // CLICKMANAGER_H

